/*
 * Copyright (c) 2012 Wayne Gray All rights reserved
 * 
 * This file is part of devboard.
 * 
 * Infinity PFM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Infinity PFM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Infinity PFM.  If not, see <http://www.gnu.org/licenses/>.
 */
$(document).ready(function() {
	$("#board_link").hide();
	$("#data_output").hide();

	var state_data = init_states($("#states").text().split("|"));
	var app_data = {
		board : init_board($("#stories").text().split("|")),
		states : state_data.states,
		states_order : state_data.states_order
	}

	$("#output").empty();
	$("#output").append(create_board(app_data));
	addPageEvents();

});

var init_states = function(states_input) {
	var states = {}
	var states_order = []
	for ( var i = 0, len = states_input.length; i < len; i++) {
		var state = states_input[i].split(",");
		if (state.length == 2) {
			states[state[0]] = state[1];
			states_order.push(state[0]);
		}
	}
	return {
		states : states,
		states_order : states_order
	};
}

var init_board = function(stories) {
	var board = {};
	for ( var i = 0, len = stories.length; i < len; i++) {
		var story = stories[i].split("^");
		var state = story[0];
		if (story.length == 4) {
			if (!board[state]) {
				board[state] = [];
			}
			board[state].push(story);
		}
	}
	return board;
}

var create_list = function(board, state) {
	var list = $("<ul class=\"state\" id=\"" + state + "\"></ul>");
	var id;

	if (board[state]) {
		for ( var i = 0, len = board[state].length; i < len; i++) {

			id = board[state][i][3];

			var story_element = $("<li id=\""
					+ id
					+ "\"><div class=\"box box_"
					+ state
					+ "\">"
					+ "<span><a style='position:relative;top:-20px; left:50%;' href='#'>.</a></span>"
					+ board[state][i][1]
					+ " "
					+ decodeURIComponent(board[state][i][2].replace(/\+/g, " "))
					+ "<br><a href='javascript:popup(\"" + id + "\")'>...</a>"
					+ "</div></li>");
			story_element.data("story", board[state][i]);
			list.append(story_element);
		}
	}
	return list
}

var create_column = function(board, state, headline) {
	var state_column = $("<div class=\"dp10"
			+ ((!/_Q$/.test(state)) ? "" : " queue_column") + "\"></div>")
	state_column.append($("<div class=\"headline\">" + headline + "</div>"));
	state_column.append(create_list(board, state));
	state_column.data("state", state);
	return state_column;
}

var create_board = function(app_data) {
	var table = $("<div id=\"board\"></div>");
	var ids = "";
	for (j = 0; j < app_data.states_order.length; j++) {
		var state = app_data.states_order[j]
		if (!/_Q$/.test(state)) {
			var queue_state = state + "_Q";
			ids += "#" + queue_state + ",";
			var queue_state_column = create_column(app_data.board, queue_state,
					app_data.states[state] + " Ready")
			table.append(queue_state_column)

			ids += "#" + state + ","
			var state_column = create_column(app_data.board, state,
					app_data.states[state])
			table.append(state_column)
		}
	}
	$(ids, table).dragsort({
		dragSelector : "li > div > span",
		dragBetween : true,
		dragEnd : dropEvent
	});
	return table;
}

function dropEvent() {

}

function getBoardOutput(withLineFeed) {

	var data = "";
	$("#board .dp10").each(
			function() {
				var state = $(this).data("state");
				$(this).find("li").each(
						function() {
							var story = $(this).data("story");
							data += state + "^" + story[1] + "^" + story[2]
									+ "^" + $(this).attr("id") + "|";
							if (withLineFeed) {
								data += "\n";
							}
						});
			});

	$("#data_output").val(data);
	return data;
}

function addPageEvents() {

	$("#board_link").click(
			function() {
				var state_data = init_states($("#states").text().split("|"));
				var app_data = {
					board : init_board($("#data_output").val().replace(
							/(\r\n|\n|\r)/gm, "").split("|")),
					states : state_data.states,
					states_order : state_data.states_order
				}

				$("#output").empty();
				$("#output").append(create_board(app_data));

				$("#output").show();
				$("#data_link").show();
				$("#data_output").hide();
				$("#board_link").hide();
			})

	$("#data_link").click(function() {
		var data = getBoardOutput(true);
		$("#data_output").val(data);
		$("#output").hide();
		$("#data_link").hide();
		$("#data_output").show();
		$("#board_link").show();
	});

	$("#add_link").click(function() {

		popup();

	});

	$("#save_link").click(function() {

		var data = getBoardOutput(false);
		$("#data_output").val(data);

		outData = data.split("|");
		inData = $("#stories").text().split("|");
		var outId, inId, inRow;

		for (i = 0; i < outData.length; i++) {

			outCols = outData[i].split("^");
			outId = outCols[3];
			inRow = "";

			for (j = 0; j < inData.length; j++) {
				inId = inData[j].split("^")[3]
				if (outId == inId) {
					inRow = inData[j];
					break;
				}
			}

			if (inRow.length > 0) {
				if (outData[i] != inRow) {

					editItem(outId, outCols[1], outCols[0], outCols[2]);

				}
			}

		}

	});

	$("#clear_link").click(
			function() {

				var data = getBoardOutput(false);
				outData = data.split("|");

				for (i = 0; i < outData.length; i++) {

					outCols = outData[i].split("^");
					outId = outCols[3];
					if (outCols[0] == "R") {
						$.ajax({
							url : "/delete/" + outId + "/" + outCols[1] + "/"
									+ outCols[0] + "/" + outCols[2],
							context : document.body,
							success : function() {
								window.location.reload();
							}
						});
					}

				}
			});

	// if user clicked on button, the overlay layer or the dialogbox, close the
	// dialog
	$('#dlgEditOk')
			.click(
					function() {

						var itemId, owner, location, description;

						itemId = $('#dlgEditIdRef').val();
						owner = "(" + $('#dlgEditOwner').val() + ")";
						description = escape(encodeURIComponent($(
								'#dlgEditDescription').val()));
						location = $('#dlgEditLocation').val();

						if (itemId.length > 0) {
							editItem(itemId, owner, location, description);
						} else {
							addItem(owner, description);
						}

						return false;
					});

	$('#dlgEditCancel').click(function() {
		$('#dialog-overlay, #dialog-box').hide();
		return false;
	});

	// if user resize the window, call the same function again
	// to make sure the overlay fills the screen and dialogbox aligned to center
	$(window).resize(function() {

		// only do it if the dialog box is not hidden
		if (!$('#dialog-box').is(':hidden'))
			popup();
	});

}

function editItem(itemId, owner, location, description) {

	if (typeof (itemId) == 'undefined')
		return;

	var editUrl = "/edit/" + itemId + "/" + owner + "/" + location + "/"
			+ description

	$.ajax({
		url : editUrl,
		context : document.body,
		success : function() {
			window.location.reload();
		}
	});

}

function addItem(owner, description) {

	if (description.length > 0 && owner.length > 0) {

		window.location = "/add/" + owner + "/P_Q/" + description;

	}

}

// Popup dialog
function popup(message) {

	if (typeof (message) != 'undefined' && message.length > 0) {
		$.get('/get/' + message, function(data) {

			var item = eval('(' + data + ')');
			$('#dlgEditId').html("Id: " + message);
			$('#dlgEditIdRef').val(message);
			$('#dlgEditOwner').val(
					item.BoardItem[0].replace(/\(/g, '').replace(/\)/g, ''));
			$('#dlgEditDescription').val(
					decodeURIComponent(item.BoardItem[2].replace(/\+/g, " ")));
			$('#dlgEditLocation').val(item.BoardItem[1]);
		});
	} else {

		$('#dlgEditId').html("");
		$('#dlgEditIdRef').val("");
		$('#dlgEditOwner').val("");
		$('#dlgEditDescription').val("");
		$('#dlgEditLocation').val("");

	}

	// get the screen height and width
	var maskHeight = $(document).height();
	var maskWidth = $(window).width();

	// calculate the values for center alignment
	// var dialogTop = ((maskHeight/3) - ($('#dialog-box').height()));
	var dialogTop = 200;
	var dialogLeft = (maskWidth / 2) - ($('#dialog-box').width() / 2);

	// assign values to the overlay and dialog box
	$('#dialog-overlay').css({
		height : maskHeight,
		width : maskWidth
	}).show();
	$('#dialog-box').css({
		top : dialogTop,
		left : dialogLeft
	}).show();

}
